<?php
/**
 * Manage all of the gift certificates on record
 */
function uc_gift_certificate_manage($view_type = 'manage') {
  $query = "SELECT gc.* ,u.name as purchasername,u2.name as username, u.mail as purchasermail,u2.mail as usermail
    FROM {uc_gift_certificates} AS gc
    left outer join {users} AS u on gc.purchaser_id = u.uid
    left outer join {users} AS u2 on gc.user_id = u2.uid";

  // $result = db_query($query);
  $rows = array();

  $header = array(
    array('data' => t('Name'), field => 'gc.name'),
    array('data' => t('Gift Certificate Purchaser'), 'field' => 'purchasername', 'width' => '230'),
    array('data' => t('Gift Certificate Recipient'), 'width' => '230', 'field' => 'username'),
    array('data' => 'Value', 'width' => '60', 'field' => 'gc.value'),
    array('data' => 'Action', 'width' => 50),
    array('sort' => 'asc'),
  );

  $result = pager_query($query.tablesort_sql($header), 200, 0, NULL);

  while ($row = db_fetch_object($result)) {
    $rows[] = array($row->name,
                    $row->purchasername." (".$row->purchasermail.")",
                    $row->username." (".$row->usermail.")",
                    $row->value,
                    l(t('edit'), "admin/store/gift_certificates/$row->certificate_id/edit") . ' ' . l(t('delete'),"admin/store/gift_certificates/$row->certificate_id/delete"),
                  );
  }

  $output = theme('table', $header, $rows).theme('pager', NULL, 20, 0);

  // No gift certificates
  if ($output == null) {
    $output = "<p>There are currently no gift certificates in the system.</p>";
  }

  return $output;
}

/**
 * Implementation of hook_form
 *
 * Show the form to add a new certificate
 */
function uc_gift_certificate_add_form($form_state, $certificate_id) {
  $is_add = $certificate_id == 'add';
  $msg_to_recipient = NULL;

  if (!$is_add) {
    $cert = uc_gift_certificate_load($certificate_id);

    $form['certificate_id'] = array(
      '#type' => 'value',
      '#value' => $cert->certificate_id,
    );
    if (!empty($cert->order_id)) {
      $order = uc_order_load($cert->order_id);
      $form['order_id'] = array(
        '#type' => 'item',
        '#value' => t('This certificate was created as part of order !order_link', array('!order_link' => l('#' . $cert->order_id, 'admin/store/orders/' . $cert->order_id))),
        '#weight' => 22,
      );
      foreach ($order->products as $product) {
        if ($product->order_product_id == $cert->order_product_id &&
            !empty($product->data['attributes']['Message To Recipient'][0])) {
          $msg_to_recipient = $product->data['attributes']['Message To Recipient'][0];
          break;
        }
      }
    }
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Gift Certificate Name'),
    '#default_value' => $cert->name,
    '#weight' => 0,
    '#required' => true,
  );
  $form['cert_code'] = array(
    '#prefix' => t('<b>Certificate Code:</b><br/>'),
    '#value' => $cert->cert_code,
    '#weight' => 14,
  );

  // If not using autocomplete
  if(!variable_get('uc_use_autocomplete_user_fields', true)) {
    $form['purchaser_id'] = array(
      '#type' => 'select',
      '#title' => t('Purchaser'),
      '#default_value' => $cert->purchaser_id,
      '#options' => uc_gift_certificate_load_users(),
      '#description' => t('Username of the purchaser of the gift certificate'),
      '#weight' => 12,
    );

    $form['user_id'] = array(
      '#type' => 'select',
      '#title' => t('Recipient'),
      '#default_value' => $cert->user_id,
      '#options' => uc_gift_certificate_load_users(),
      '#description' => t('Username of the recipient of the gift certificate'),
      '#weight' => 13,
    );
  }
  else {
    $form['purchaser_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Purchaser'),
      '#default_value' => uc_gift_certificate_username_from_uid($cert->purchaser_id),
      '#description' => t('Username of the purchaser of gift certificate'),
      '#autocomplete_path' => 'user/autocomplete',
      '#weight' => 12,
    );

    $form['user_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Recipient'),
      '#default_value' => uc_gift_certificate_username_from_uid($cert->user_id),
      '#autocomplete_path' => 'user/autocomplete',
      '#description' => t('Username of the recipient of the gift certificate'),
      '#weight' => 13,
    );
  }
  $form['value'] = array(
    '#type' => 'textfield',
    '#title' => t('Value'),
    '#default_value' => $cert->value,
    '#size' => 25,
    '#description' => t('Amount this gift certificate is worth.'),
    '#required' => true,
    '#weight' => 6,
  );


  if ($is_add) {
    $form['new_user'] = array(
      '#type' => 'fieldset',
      '#title' => t('Create a new user'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#weight' => 18,
    );
    $form['new_user']['new_user_checkbox'] = array(
      '#type' => 'checkbox',
      '#title' => t('Create a new user account for the recipient'),
      '#description' => t('Please check here to create a new user account for the recipient. You must then enter the email of the new user below.'),
      '#weight' => 18,
    );
    $form['new_user']['new_user_email'] = array(
      '#type' => 'textfield',
      '#size' => 25,
      '#title' => t('New user\'s email address'),
      '#description' => t(''),
      '#weight' => 19,
    );
  }
  $form['msg_to_recipient'] = array(
    '#type' => 'textarea',
    '#title' => t('Message to recipient'),
    '#default_value' => $msg_to_recipient,
    '#weight' => 20,
  );
  $form['mail_user_checkbox'] = array(
    '#type' => 'checkbox',
    '#title' => t('Email the recipient'),
    '#description' => t('Check here to email the recipient the gift certificate.'),
    '#weight' => 21,
  );
  $form['cancel'] = array(
    '#type' => 'item',
    '#value' => l(t('Cancel and return to Gift Certificate Listing'), 'admin/store/gift_certificates'),
    '#weight' => 25,
  );
  $form['op'] = array('#type' => 'submit',
    '#value' => t('Submit'),
    '#weight' => 26,
  );
  return $form;
}

/**
 * Implementation of hook_form_validate()
 */
function uc_gift_certificate_add_form_validate($form, &$form_state) {
  $use_auto = variable_get('uc_use_autocomplete_user_fields', true);
  $is_new_user = $form_state['values']['new_user_checkbox'];
  if ($use_auto) {
    $form_state['values']['purchaser_id'] = uc_gift_certificate_uid_from_username($form_state['values']['purchaser_id']);
    $form_state['values']['user_id'] =  uc_gift_certificate_uid_from_username($form_state['values']['user_id']);
  }

  if (!db_result(db_query('SELECT uid FROM {users} WHERE uid != 0 AND uid = %d', $form_state['values']['purchaser_id']))) {
    form_set_error('purchaser_id', t('Purchaser is not a valid user.'));
  }
  if (!$is_new_user && !db_result(db_query('SELECT uid FROM {users} WHERE uid != 0 AND uid = %d', $form_state['values']['user_id']))) {
    form_set_error('user_id', t('Recipient is not a valid user.'));
  }
  if (empty($form_state['values']['value']) || !is_numeric($form_state['values']['value'])) {
    form_set_error('value', t('Value is not a valid amount.'));
  }
}

/**
 * Implementation of hook_form_submit()
 * Adding a new gift certificate
 */
function uc_gift_certificate_add_form_submit($form, &$form_state) {
  $use_auto = variable_get('uc_use_autocomplete_user_fields', true);
  $is_new_cert = !isset($form_state['values']['certificate_id']);
  $is_new_user = $form_state['values']['new_user_checkbox'];
  $do_mail = $form_state['values']['mail_user_checkbox'];

  //drupal_set_message($form['cert_code']);
  // Make sure we don't generate a new cert code if we are editing an existing certificate.
  if (!isset($form_state['values']['cert_code'])) {
    $cert_code = uc_gift_certificate_mKey();
  }
  else {
    $cert_code = $form_state['values']['cert_code'];
  }

  if ($is_new_cert) {
    $success = drupal_write_record('uc_gift_certificates', $form_state['values']);
    $certificate_id = db_last_insert_id('uc_gift_certificates', 'certificate_id');
    $cert_code = $certificate_id."-".$cert_code;
    db_query("UPDATE {uc_gift_certificates} SET cert_code = '%s' WHERE certificate_id = %d", $cert_code, $certificate_id);
  } else {
    $certificate_id = $form_state['values']['certificate_id'];
    $success = drupal_write_record('uc_gift_certificates', $form_state['values'], 'certificate_id');
  }

  if ($success) {
    if ($do_mail) {
      if ($is_new_user) {
        uc_gift_certificate_mail_cert_notice($form_state['values']['new_user_email'], $certificate_id, $form_state['values']['msg_to_recipient']);
        drupal_set_message(t('A new user was created and emailed their gift certificate.'));
      }
      else {
        $cert = uc_gift_certificate_load($certificate_id);
        $user = user_load(array('uid' => $cert->user_id));
        uc_gift_certificate_mail_cert_notice($user->mail, $certificate_id, $form_state['values']['msg_to_recipient']);
        drupal_set_message(t('The recipient was emailed their gift certificate.'));
      }
    }

    if ($is_new_cert) {
      drupal_set_message('Successfully added the gift certificate.');
    } else {
      drupal_set_message('Successfully updated the certificate.');
    }
  }
  drupal_goto('admin/store/gift_certificates');
}

/**
 * Confirm certificate deletion
 */
function uc_gift_certificate_delete_confirm($form_state, $cid) {
 $form['cid'] = array('#type' => 'value', '#value' => $cid);

 return confirm_form($form,
  t('Confirm Deletion of Gift Certificate %cid', array('%cid' => $cid)),
    'admin/store/gift_certificates',
  t('Deleting this gift certificate is irreversible.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Delete gift certificate
 */
function uc_gift_certificate_delete_confirm_submit($form, &$form_state) {
  $cert_id = $form_state['values']['cid'];
  $query = "DELETE FROM {uc_gift_certificates} WHERE certificate_id = '" . $cert_id . "'";
  if (db_query($query)) {
    drupal_set_message("Gift Certificate Deleted Successfully");
  }
  else {
    drupal_set_message("Error deleting gift certificate");
  }
  $form_state['redirect'] = 'admin/store/gift_certificates';
}

/**
 * Hook_settings
 */
function uc_gift_certificate_settings() {
  $form['email_first']['uc_gc_email_new_user_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Email Subject - New User'),
    '#default_value' => variable_get('uc_gc_email_new_user_subject', uc_gift_certificate_get_setting_defaults('uc_gc_email_new_user_subject')),
    '#description' => t('Email sent to notify a new user. Available variables are: !gift_certificate_message, !gift_certificate_value, !gift_certificate_value_integer, !gift_certificate_code, !username, !site, !password, !uri, !uri_brief, !mailto, !date, !login_uri, !edit_uri, !login_url.'),
  );
  $form['email_first']['uc_gc_email_new_user_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Email Body'),
    '#default_value' => variable_get('uc_gc_email_new_user_body', uc_gift_certificate_get_setting_defaults('uc_gc_email_new_user_body')),
    '#description' => t('Email sent to notify an existing user.'),
  );
  $form['email_second']['uc_gc_email_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Email Subject - Existing User'),
    '#default_value' => variable_get('uc_gc_email_subject', uc_gift_certificate_get_setting_defaults('uc_gc_email_subject')),
    '#description' => t('Email sent to notify an existing user. Available variables are: !gift_certificate_message, !gift_certificate_value, !gift_certificate_value_integer, !gift_certificate_code, !username, !site,  !uri, !uri_brief, !mailto, !date, !login_uri.'),
  );
  $form['email_second']['uc_gc_email_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Email Body'),
    '#default_value' => variable_get('uc_gc_email_body', uc_gift_certificate_get_setting_defaults('uc_gc_email_body')),
    '#description' => t('Email sent to notify an existing user.'),
  );
  $form['uc_gc_email_format'] = array(
    '#type' => 'select',
    '#title' => t('Email format'),
    '#options' => _uc_gift_certificate_format_options(),
    '#default_value' => variable_get('uc_gc_email_format', uc_gift_certificate_get_setting_defaults('uc_gc_email_format')),
    '#description' => t('If you have the mimemail module installed, you can choose to send your email messages in HTML format.'),
  );
  $form['uc_gc_pane_msg'] = array(
    '#type' => 'textarea',
    '#title' => t('Checkout Pane Message'),
    '#default_value' => variable_get('uc_gc_pane_msg', uc_gift_certificate_get_setting_defaults('uc_gc_pane_msg')),
    '#description' => t('Gift certificate message in the checkout pane.'),
  );
  $form['uc_use_autocomplete_user_fields'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use Autocomplete Fields'),
    '#default_value' => variable_get('uc_use_autocomplete_user_fields', true),
    '#description' => t('Use autocomplete fields for choosing which users are associated with a given gift certificate.'),
  );
  foreach (uc_order_status_list('general') as $status) {
    $options[$status['id']] = $status['title'];
  }
  foreach (uc_order_status_list('specific') as $status) {
    $options[$status['id']] = $status['title'];
  }
  $form['uc_gift_certificate_order_status_issue'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Order Status for Issuing'),
    '#default_value' => variable_get('uc_gift_certificate_order_status_issue', uc_gift_certificate_get_setting_defaults('uc_gift_certificate_order_status_issue')),
    '#options' => $options,
    '#attributes' => array('style' => 'display: inline;'),
    '#description' => t('Choose what the order status must be in order the for the gift certificate to be issued (upon purchase).'),
  );
  $form['uc_gift_certificate_order_status_deduct'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Order Status for Deducting'),
    '#default_value' => variable_get('uc_gift_certificate_order_status_deduct', uc_gift_certificate_get_setting_defaults('uc_gift_certificate_order_status_deduct')),
    '#options' => $options,
    '#attributes' => array('style' => 'display: inline;'),
    '#description' => t('Choose what the order status must be in order the for the gift certificate to be deducted (upon using it).'),
  );
  return system_settings_form($form);
}

/**
 * Issue a new gift certificate
 */
function uc_gift_certificate_issue() {
  global $user;

  $num_rows = db_result(db_query("SELECT COUNT(*) FROM {uc_gift_certificates} WHERE purchaser_id = %d", $user->uid));
  $result = db_query("SELECT * FROM {uc_gift_certificates} WHERE purchaser_id = %d", $user->uid);
  $rows = array();

  while ($row = db_fetch_object($result)) {
    if ($num_rows == 1) {
      drupal_goto("gift_certificates/$row->certificate_id/issue");
    }
    $rows[] = array($row->name, $row->value, l(t('send'), "gift_certificates/$row->certificate_id/issue"));
  }
  $header = array(array('data' => 'Name', 'width' => '100'),  array('data' => 'Value', 'width' => '60'),  array('data' => '', 'width' => 50));
  $output .= theme('table',$header, $rows);

  if ($output == null) {
    $output = "<p>There are currently no gift certificates in the system.</p>";
  }
  return $output;
}

/**
 * Determine possible mail format options.
 *
 * The mime_mail module must be installed to send HTML mails.
 */
function _uc_gift_certificate_format_options() {
  $options = array('plain' => t('plain'));
  if (module_exists('mimemail')) {
    $options['html'] = t('html');
  }
  return $options;
}
